from abc import ABC, abstractmethod
import json
import logging
import requests
from .exc import TransferError

LOG = logging.getLogger("wetransfer")
LOG.addHandler(logging.NullHandler())
LOG.setLevel(logging.INFO)


def http_response(func):
    def wrapper_http_response(*args, **kwargs):
        """
        The wrapper calls the original function, then uses the response object
        to create a <status_code, body> tuple for further use.
        If a keyword argument called 'status' is found, it is used to check the
        status code. If the actual status code does not equal the expected one,
        an error will be raised. Used for get, put, post.
        """
        expected_status = None
        if 'status' in kwargs:
            expected_status = kwargs['status']
            del kwargs['status']

        r = func(*args, **kwargs)
        LOG.debug(r.text)
        status = r.status_code
        body = json.loads(r.text)
        if expected_status is not None and expected_status != status:
            LOG.error(status, body['message'])
            raise TransferError('%d: %s' % (status, body['message']))

        return status, body

    return wrapper_http_response


class HttpClient(ABC):
    def __init__(self):
        super().__init__()

    @abstractmethod
    def authorization_headers(self):
        pass

    @abstractmethod
    def endpoint(self, address):
        pass

    @http_response
    def get(self, address):
        """
        convenience method to GET
        :param address: URL endpoint
        :return: response object
        """
        headers = self.authorization_headers()
        LOG.info('GET Address: %s' % address)
        LOG.debug('Headers: %s' % headers)
        return requests.get(self.endpoint(address), headers=headers)

    @http_response
    def post(self, address, **kwargs):
        """from wetransfer.exc import WeTransferError

        Convenience method to POST
        :param address: URL endpoint
        :param kwargs: headers, data, etc.
        :return: response object
        """
        kwargs['headers'] = self.authorization_headers()
        LOG.info('POST Address: %s' % address)
        LOG.debug('Headers: %s' % kwargs['headers'])
        return requests.post(self.endpoint(address), **kwargs)

    @http_response
    def put(self, address, **kwargs):
        """
        Convenience method to PUT
        :param address: URL endpoint
        :param kwargs: headers
        :return: response object
        """
        kwargs['headers'] = self.authorization_headers()
        LOG.info('PUT Address: %s' % address)
        LOG.debug('Headers: %s' % kwargs['headers'])
        return requests.put(self.endpoint(address), **kwargs)



import os
import subprocess
from unittest import TestCase
from wetransfer import TransferApi
from wetransfer.exc import TransferError

image_dir = './wetransfer/tests/images/'
images = ['we_483_lines.jpg', 'we_nytimes_copy.jpg']  # actual WeTransfer images


class TestTransfer(TestCase):
    def setUp(self):
        try:
            self.transfer_api = TransferApi(os.environ['WE_API_KEY'])

        except KeyError:
            self.skipTest('Oops. You forgot to set the WE_API_KEY.')

    def test_single_file_upload(self):
        try:
            filepath = image_dir + images[0]
            self.transfer_api.upload_file('Upload an image', filepath)

        except TransferError as e:
            self.fail('Oops. Uploading an image caused to API to raise an error: %s' % e)

    def test_multi_file_upload(self):
        try:
            filepaths = [image_dir + x for x in images]
            self.transfer_api.upload_files('Upload multiple images', filepaths)

        except TransferError as e:
            self.fail('Oops. Uploading an image caused to API to raise an error: %s' % e)

    def test_large_file_upload(self):
        def run_command(cmd, file_name):
            cmd = cmd % file_name
            subprocess.run(cmd.split(' '))

        filename = '/tmp/we_rnd.bin'
        create_random_file_sh = 'dd if=/dev/urandom of=%s count=6144 bs=1024 iflag=fullblock status=none'
        delete_random_file_sh = 'rm -rf %s'
        try:
            run_command(create_random_file_sh, filename)

            self.transfer_api.upload_file('Upload a rather large file', filename)

        except TransferError as e:
            self.fail('Oops. Uploading an image caused to API to raise an error: %s' % e)

        finally:
            run_command(delete_random_file_sh, filename)



import json
import logging
from .base import WeTransferBase
from .file import File

LOG = logging.getLogger("wetransfer")
LOG.addHandler(logging.NullHandler())
LOG.setLevel(logging.INFO)


class WeTransfer(WeTransferBase):
    WE_ENDPOINT_DEV = 'https://dev.wetransfer.com'

    def __finalize_transfer(self, transfer_id):
        """
        Finalize transfer.
        :param transfer_id: transfer id.
        :return: WeTransfer URL
        """
        _, body = self.put('transfers/%s/finalize' % transfer_id, status=200)
        return body['url']

    def __complete_file_upload(self, transfer_id, file_id, part_numbers):
        """
        Complete file upload.
        :param transfer_id: transfer id
        :param file_id: file id
        :param part_numbers: part numbers
        :return: None
        """
        data = {'part_numbers': part_numbers}
        LOG.debug(json.dumps(data, sort_keys=True, indent=2, separators=(',', ': ')))
        self.put('transfers/%s/files/%s/upload-complete' % (transfer_id, file_id),
                 data=json.dumps(data), status=200)

    def __request_upload_url(self, transfer_id, file_id, part_number):
        """
        Request special upload url, which is tailored for AWS S3
        :param transfer_id: transfer id
        :param file_id: file id
        :param part_number: part number
        :return: AWS S3 upload url
        """
        _, body = self.get('transfers/%s/files/%s/upload-url/%s' % (transfer_id, file_id, part_number), status=200)
        return body['url']

    def __create_transfer(self, message, files):
        """
        Create a new transfer.
        :param message: Message that goes with the transfer
        :param files: An array of files
        :return:
        """
        files_stream = [{'name': file.name, 'size': file.size} for file in files]
        data = {'message': message, 'files': files_stream}

        _, body = self.post('transfers', data=json.dumps(data), status=201)

        LOG.debug(json.dumps(body, sort_keys=True, indent=2, separators=(',', ': ')))

        files_info = body['files']
        for i in range(len(files_info)):
            file_info = files_info[i]
            multipart = file_info['multipart']

            file = files[i]
            file.id = file_info['id']
            file.part_numbers = multipart['part_numbers']
            file.chunk_size = multipart['chunk_size']

        return body['id']

    def upload_files(self, message, filepaths):
        """
        Main entrypoint for this class. Pass in a message and a list of filepaths to upload.
        :param message: Message to go with uploads
        :param filepaths: A list of filepaths of files to upload
        :return: The download URL generated by WeTransfer
        """
        files = [File(filepath) for filepath in filepaths]
        transfer_id = self.__create_transfer(message, files)
        for file in files:
            part_number = 1
            with open(file.path, 'rb') as fh:
                while True:
                    bytes_read = fh.read(file.chunk_size)
                    if not bytes_read:  # empty string?
                        break

                    url = self.__request_upload_url(transfer_id, file.id, part_number)
                    self.s3_file_upload(url, bytes_read)
                    part_number += 1

            self.__complete_file_upload(transfer_id, file.id, file.part_numbers)

        return self.__finalize_transfer(transfer_id)

    def upload_file(self, message, file_path):
        """
        Upload a single file.
        :param message: Message
        :param file_path: Path of file to upload
        :return: The download URL generated by WeTransfer
        """
        return self.upload_files(message, [file_path])



import json
import logging
import requests
from .exc import TransferError
from .http import HttpClient


LOG = logging.getLogger("wetransfer")
LOG.addHandler(logging.NullHandler())
LOG.setLevel(logging.INFO)


class WeTransferBase(HttpClient):
    WE_ENDPOINT_DEV = 'https://dev.wetransfer.com'

    def __init__(self, api_key, user_identifier=None):
        super().__init__()

        self.__x_api_key = api_key
        self.__user_identifier = user_identifier

        self.__token = None

        self.__authorize()

    def __authorize(self):
        """
        Authorize user based on x-api-key only. Store returned bearer token for future use.
        :return: None
        """
        address = self.endpoint('authorize')
        headers = self.__base_authorization_headers()
        kwargs = {'headers': headers}
        if self.__user_identifier is not None:
            kwargs['data'] = {'user_identifier': self.__user_identifier}

        r = requests.post(address, **kwargs)
        body = json.loads(r.text)
        if r.status_code != 200:
            LOG.error(body['message'])
            raise TransferError(body['message'])

        self.__token = 'Bearer %s' % body['token']

    def endpoint(self, method):
        """
        Get endpoint for certain API method. Version v2.
        :param method: method to append to endpoint
        :return: endpoint for method
        """
        return '%s/%s/%s' % (self.WE_ENDPOINT_DEV, 'v2', method)

    def __base_authorization_headers(self):
        """
        Get base authorization headers.
        :return: headers
        """
        headers = {
            "Content-Type": "application/json",
            "x-api-key": self.__x_api_key
        }
        return headers

    def authorization_headers(self):
        """
        Get authorization headers.
        :return: headers
        """
        base_headers = self.__base_authorization_headers()
        headers = {'Authorization': self.__token}
        return {**headers, **base_headers}

    def is_authenticated(self):
        """
        Indicate that we're authenticated (or not).
        :return: True if authenticated, False otherwise
        """
        return self.__token is not None

    @staticmethod
    def s3_file_upload(url, filedata):
        """
        Convenience function to explicitly upload files to S3
        :param url: S3 endpoint
        :param filedata: actual data
        :return: None
        """
        r = requests.put(url, data=filedata)
        if r.status_code != 200:
            LOG.error(r.text)
            raise TransferError('Error uploading file(s) to AWS S3.')



import os
from unittest import TestCase
from wetransfer import TransferApi


class TestAuthentication(TestCase):
    def setUp(self):
        try:
            self.transfer_api = TransferApi(os.environ['WE_API_KEY'])

        except KeyError:
            self.skipTest('Oops. You forgot to set the WE_API_KEY.')

    def test_authentication(self):
        self.assertTrue(self.transfer_api.is_authenticated())



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py3-wetransfer",
    version="1.0.0",
    author="Sander Huijsen",
    author_email="sander.huijsen@gmail.com",
    maintainer="Sander Huijsen",
    maintainer_email="sander.huijsen@gmail.com",
    description="A Python 3 wrapper to use WeTransfer API V2 transfer and board",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Sandyman/py3-wetransfer",
    packages=setuptools.find_packages(),
    install_requires=[
        "requests>=2.7.0"
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP"
    ],
    test_suite='nose.collector',
    tests_require=['nose'],
)



import json
import logging
from .base import WeTransferBase
from .file import File

LOG = logging.getLogger("wetransfer")
LOG.addHandler(logging.NullHandler())
LOG.setLevel(logging.INFO)


class Board(WeTransferBase):

    def get_board(self, board_id):
        """
        Get a board based on board_id
        :param board_id: board id
        :return: board information
        """
        _, body = self.get('boards/%s' % board_id, status=200)
        return body

    def create_new_board(self, name):
        """
        Create a new board
        :param name: Name of new board
        :return: id, url tuple
        """
        data = {'name': name}
        LOG.debug(json.dumps(data, sort_keys=True, indent=4, separators=(',', ': ')))
        _, body = self.post('boards', data=json.dumps(data), status=201)
        return body['id'], body['url']

    def add_links_to_board(self, board_id, data):
        """
        Add links to a board. The data must be a list containing of link objects, with
        a link object being a dict with a title and a url.
        :param board_id: Board id
        :param data: Data
        :return: Board info (see get_board())
        """
        self.post('boards/%s/links' % board_id, data=json.dumps(data), status=201)
        return self.get_board(board_id)

    def __complete_file_upload_board(self, board_id, file_id):
        """
        Mark the file upload as complete
        :param board_id: Board id
        :param file_id: File id
        :return: None
        """
        self.put('boards/%s/files/%s/upload-complete' % (board_id, file_id), status=202)

    def __request_upload_url_board(self, board_id, file_id, part_number, multipart_upload_id):
        """
        Request special upload url, which is tailored for AWS S3
        :param board_id: Board id
        :param file_id: File id
        :param part_number: part number
        :param multipart_upload_id: multipart upload id
        :return: url
        """
        _, body = self.get('boards/%s/files/%s/upload-url/%s/%s' %
                           (board_id, file_id, part_number, multipart_upload_id),
                           status=200)
        return body['url']

    def add_files_to_board(self, board_id, filepaths):
        """
        Add files to board
        :param board_id: Board id
        :param filepaths: list of filepaths
        :return:
        """
        files = [File(filepath) for filepath in filepaths]
        data = [{'name': file.name, 'size': file.size} for file in files]
        LOG.debug(json.dumps(data, sort_keys=True, indent=4, separators=(',', ': ')))
        _, body = self.post('boards/%s/files' % board_id, data=json.dumps(data), status=201)

        LOG.debug(json.dumps(body, sort_keys=True, indent=4, separators=(',', ': ')))

        upload_plans = body
        for i in range(len(upload_plans)):
            file = files[i]
            upload_plan = upload_plans[i]
            multipart = upload_plan['multipart']
            part_number = 1
            with open(file.path, 'rb') as fh:
                while True:
                    bytes_read = fh.read(multipart['chunk_size'])
                    if not bytes_read:  # empty string?
                        break

                    url = self.__request_upload_url_board(board_id, upload_plan['id'], part_number, multipart['id'])
                    self.s3_file_upload(url, bytes_read)
                    part_number += 1

            self.__complete_file_upload_board(board_id, upload_plan['id'])

        return self.get_board(board_id)



import os


class File(object):
    """
    Convenience class for file objects
    """
    def __init__(self, filepath):
        self.path = filepath
        self.name = os.path.basename(filepath)
        self.size = os.path.getsize(filepath)

        self.id = None
        self.part_numbers = 0
        self.chunk_size = 0



import os
import unittest
from wetransfer import BoardApi
from wetransfer.exc import TransferError

image_dir = './wetransfer/tests/images/'
images = ['we_483_lines.jpg', 'we_nytimes_copy.jpg']  # actual WeTransfer images

unittest.TestLoader.sortTestMethodsUsing = None


class TestBoard(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """
        We only want to create a single board for all tests in this file, which is why
        we use the classmethod setUpClass.
        """
        cls.board_id = None

        try:
            cls.board_api = BoardApi(os.environ['WE_API_KEY'])
        except KeyError:
            super().skipTest('Oops. You forgot to set the WE_API_KEY.')

        try:
            board_id, board_url = cls.board_api.create_new_board('test board')
            cls.board_id = board_id

        except TransferError as e:
            super().fail('Oops. Uploading an image caused to API to raise an error: %s' % e)

    def test_add_links(self):
        links = [{
            'title': 'Google',
            'url': 'https://google.com',
        }, {
            'title': 'WeTransfer',
            'url': 'https://wetransfer.com',
        }]

        try:
            self.board_api.add_links_to_board(self.board_id, links)

        except TransferError as e:
            self.fail('Oops. Adding links to board caused API to raise an error: %s' % e)

    def test_add_files(self):
        filepaths = [image_dir + x for x in images]

        try:
            self.board_api.add_files_to_board(self.board_id, filepaths)

        except TransferError as e:
            self.fail('Oops. Adding files to board caused API to raise an error: %s' % e)

    def test_get_board(self):
        try:
            board_info = self.board_api.get_board(self.board_id)

            self.assertEqual('test board', board_info['name'])

            items = board_info['items']
            self.assertEqual(4, len(items), 'Oops. Expected a total of 4 items.')

            links = [x for x in items if x['type'] == 'link']
            files = [x for x in items if x['type'] == 'file']

            self.assertEqual(2, len(links), 'Oops. Expected a total of 2 links.')
            self.assertEqual(2, len(files), 'Oops. Expected a total of 2 files.')

        except TransferError as e:
            self.fail('Oops. Getting board info caused API to raise an error: %s' % e)
